import numpy as np
import pandas as pd


def _apply_on_bank(match_info):
    for pref in ['STAKE_', 'PAYOUT_']:
        for suff in ['ON_HOME', 'ON_FAVORITE', 'ON_DOG']:
            match_info[pref + suff] = 0.0
    match_info['BANK'] = -1.0
    for col in ['ON_HOME', 'ON_FAVORITE', 'ON_DOG']:
        cols = ['RESULT_{}'.format(col), 'ODDS_{}'.format(col),
                'STAKE_{}'.format(col), 'PAYOUT_{}'.format(col)]
        vals = match_info[['RESULT'] + cols].values
        #     print(vals.shape)
        BANK, BANK_PERCENTAGE = 1000, 0.01
        for iRow in range(len(vals)):
            _stake = BANK * BANK_PERCENTAGE
            vals[iRow][3] = _stake
            vals[iRow][4] = float(np.where(
                vals[iRow][0] == vals[iRow][1],
                vals[iRow][2] * vals[iRow][3],
                0
            ))
            BANK = BANK - _stake + vals[iRow][4]
            #     print(len(cols), match_info[cols].shape, vals[:, 1:].shape)
        match_info.loc[:, cols] = vals[:, 1:]

    return match_info


def apply_finance_strategy(match_info, strategy='FLAT'):
    if strategy == 'BANK':
        match_info = _apply_on_bank(match_info)
    else:
        FLAT_STAKE = 10
        DESIRE_PROFIT = 20
        if strategy == 'FLAT':
            match_info['STAKE_ON_HOME'] = FLAT_STAKE
            match_info['STAKE_ON_FAVORITE'] = FLAT_STAKE
            match_info['STAKE_ON_DOG'] = FLAT_STAKE
        else:
            match_info['STAKE_ON_HOME'] = float(DESIRE_PROFIT) / (match_info.ODDS_ON_HOME - 1)
            match_info['STAKE_ON_FAVORITE'] = float(DESIRE_PROFIT) / (match_info.ODDS_ON_FAVORITE - 1)
            match_info['STAKE_ON_DOG'] = float(DESIRE_PROFIT) / (match_info.ODDS_ON_DOG - 1)

        match_info['PAYOUT_ON_HOME'] = match_info.ODDS_ON_HOME * match_info.STAKE_ON_HOME
        match_info.loc[~(match_info.RESULT_ON_HOME == match_info.RESULT), 'PAYOUT_ON_HOME'] = 0

        match_info['PAYOUT_ON_FAVORITE'] = match_info.ODDS_ON_FAVORITE * match_info.STAKE_ON_FAVORITE
        match_info.loc[~(match_info.RESULT_ON_FAVORITE == match_info.RESULT), 'PAYOUT_ON_FAVORITE'] = 0

        match_info['PAYOUT_ON_DOG'] = match_info.ODDS_ON_DOG * match_info.STAKE_ON_DOG
        match_info.loc[~(match_info.RESULT_ON_DOG == match_info.RESULT), 'PAYOUT_ON_DOG'] = 0

    return match_info


import sqlite3
import pandas as pd

from betalytics.soccer.const import BOOKIE, MATCH_INFO


def load_data(db_file_name, query="SELECT * FROM football_data"):
    with sqlite3.connect(db_file_name) as con:
        all_data = pd.read_sql_query(query, con)

    all_data.columns = [col.upper() for col in all_data.columns]
    return all_data


def load_and_normalize_data(db_file_name, bookie='BET365'):
    match_info = load_data(db_file_name)

    columns_to_rename = {
        'HOMETEAM': 'HOME_TEAM',
        'AWAYTEAM': 'AWAY_TEAM',
        'FTHG': 'HOME_TEAM_GOAL',
        'FTAG': 'AWAY_TEAM_GOAL',
        'FTR': 'RESULT'
    }
    columns_to_rename.update(BOOKIE[bookie])

    match_info = match_info[MATCH_INFO + list(BOOKIE[bookie].keys())].rename(columns=columns_to_rename)

    match_info = match_info.sort_values(by='DATE', ascending=True)
    print('Dataset[Full] Shape = {}'.format(match_info.shape))
    match_info = match_info.dropna()
    match_info = match_info[
        (match_info.HOME_ODDS >= 1.0) &
        (match_info.DRAW_ODDS >= 1.0) &
        (match_info.AWAY_ODDS >= 1.0)
        ]

    match_info.loc[:, 'LEAGUE'] = match_info['COUNTRY'] + '.' + match_info['LEAGUE']
    match_info.loc[:, 'HOME_TEAM'] = match_info['DIV'] + '.' + match_info['HOME_TEAM']
    match_info.loc[:, 'AWAY_TEAM'] = match_info['DIV'] + '.' + match_info['AWAY_TEAM']

    print('Dataset[DropNA] Shape = {}'.format(match_info.shape))

    return match_info


if __name__ == '__main__':
    load_and_normalize_data(db_file_name=r'd:\WORK\Research\Betting\football_data\database.sqlite')


import unittest

from betalytics.soccer.loader.football_data_loader import load_and_normalize_data
from betalytics.common.test_utils import ordered


class TestSoccerDataLoader(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestSoccerDataLoader, self).__init__(*args, **kwargs)
        self.db = None

    @ordered
    def test_initial_load(self):
        """
        Test the Loader loads data properly
        """
        self.db = load_and_normalize_data(db_file_name=r'../../data/database.sqlite', bookie='BET365')
        self.assertEqual(self.db.shape[1], 13)

    @ordered
    def test_todo(self):
        """

        """
        pass


if __name__ == '__main__':
    unittest.main()



import pandas as pd


def apply_result_on_favorite(x):
    #     print(x.values)
    _result, _home, _draw, _away = x

    if (min(_home, _draw, _away) == _home) and (_result == 'H'):
        return 'H'
    elif (min(_home, _draw, _away) == _draw) and (_result == 'D'):
        return 'D'
    elif (min(_home, _draw, _away) == _away) and (_result == 'A'):
        return 'A'
    else:
        return 'OTHER'


def apply_result_on_dog(x):
    #     print(x.values)
    _result, _home, _draw, _away = x

    if (max(_home, _draw, _away) == _home) and (_result == 'H'):
        return 'H'
    elif (max(_home, _draw, _away) == _draw) and (_result == 'D'):
        return 'D'
    elif (max(_home, _draw, _away) == _away) and (_result == 'A'):
        return 'A'
    else:
        return 'OTHER'


def apply_results(match_info):
    # compute the match result (i.e Home win/Draw/Away win) from the goals data for the match
    match_info['RESULT'] = 'H'
    match_info.loc[match_info.HOME_TEAM_GOAL == match_info.AWAY_TEAM_GOAL, "RESULT"] = 'D'
    match_info.loc[match_info.HOME_TEAM_GOAL < match_info.AWAY_TEAM_GOAL, "RESULT"] = 'A'

    match_info.loc[:, 'ODDS_ON_HOME'] = match_info['HOME_ODDS']
    match_info.loc[:, 'ODDS_ON_FAVORITE'] = match_info[['HOME_ODDS', 'DRAW_ODDS', 'AWAY_ODDS']].min(axis=1)
    match_info.loc[:, 'ODDS_ON_DOG'] = match_info[['HOME_ODDS', 'DRAW_ODDS', 'AWAY_ODDS']].max(axis=1)

    # find the match outcome corresponding to the safest & riskiest odds
    match_info.loc[:, 'RESULT_ON_HOME'] = 'OTHER'
    match_info.loc[match_info.RESULT == 'H', 'RESULT_ON_HOME'] = 'H'

    match_info.loc[:, 'RESULT_ON_FAVORITE'] = match_info[['RESULT', 'HOME_ODDS', 'DRAW_ODDS', 'AWAY_ODDS']].apply(
        apply_result_on_favorite,
        axis='columns'
    )

    match_info.loc[:, 'RESULT_ON_DOG'] = match_info[['RESULT', 'HOME_ODDS', 'DRAW_ODDS', 'AWAY_ODDS']].apply(
        apply_result_on_dog,
        axis='columns'
    )
    return match_info


def apply_results_to_teams(match_info, teams):
    match_info.loc[:, 'BET_ON_TEAM'] = 'OTHER'
    match_info.loc[match_info.HOME_TEAM.isin(teams), 'BET_ON_TEAM'] = match_info.HOME_TEAM
    match_info.loc[match_info.AWAY_TEAM.isin(teams), 'BET_ON_TEAM'] = match_info.AWAY_TEAM

    match_info.loc[:, 'RESULT_ON_TEAM'] = 'OTHER'
    match_info.loc[
        (match_info.HOME_TEAM.isin(teams) & (match_info.RESULT == 'H')) |
        (match_info.AWAY_TEAM.isin(teams) & (match_info.RESULT == 'A')),
        'RESULT_ON_TEAM'] = match_info.RESULT

    match_info.loc[:, 'ODDS_ON_TEAM'] = 'OTHER'
    match_info.loc[(match_info.HOME_TEAM.isin(teams) & (match_info.RESULT == 'H')), 'ODDS_ON_TEAM'] = match_info.B365H
    match_info.loc[(match_info.AWAY_TEAM.isin(teams) & (match_info.RESULT == 'A')), 'ODDS_ON_TEAM'] = match_info.B365A

    return match_info



import collections

from itertools import chain, combinations


def powerset(iterable, comb_total=None):
    "powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)"
    s = list(iterable)
    if comb_total:
        return chain.from_iterable(combinations(s, r) for r in range(comb_total, len(s) + 1))
    else:
        return chain.from_iterable(combinations(s, r) for r in range(len(s) + 1))


def count_frequency(arr):
    return collections.Counter(arr)



import unittest


def make_orderer():
    order = {}

    def ordered(f):
        order[f.__name__] = len(order)
        return f

    def compare(a, b):
        return [1, -1][order[a] < order[b]]

    return ordered, compare


ordered, compare = make_orderer()
unittest.defaultTestLoader.sortTestMethodsUsing = compare


import pandas as pd

from betalytics.soccer.finance_management.all import apply_finance_strategy


def get_stats_from_matches(match_info, need_print=False):
    if match_info.shape[0] == 0:
        return None, None, None
    percent_on_home = match_info[~(match_info.PAYOUT_ON_HOME == 0)].shape[0] / float(match_info.shape[0])
    percent_on_favorite = match_info[~(match_info.PAYOUT_ON_FAVORITE == 0)].shape[0] / float(match_info.shape[0])
    percent_on_dog = match_info[~(match_info.PAYOUT_ON_DOG == 0)].shape[0] / float(match_info.shape[0])

    investment_on_home = match_info.STAKE_ON_HOME.sum()
    investment_on_favorite = match_info.STAKE_ON_FAVORITE.sum()
    investment_on_dog = match_info.STAKE_ON_DOG.sum()

    roi_on_home = sum(match_info.PAYOUT_ON_HOME)
    roi_on_favorite = sum(match_info.PAYOUT_ON_FAVORITE)
    roi_on_dog = sum(match_info.PAYOUT_ON_DOG)
    # calculate percantage reduction error what we stake vs earcn in %
    pre_on_home = (roi_on_home - investment_on_home) / investment_on_home * 100
    pre_on_favorite = (roi_on_favorite - investment_on_favorite) / investment_on_favorite * 100
    pre_on_dog = (roi_on_dog - investment_on_dog) / investment_on_dog * 100
    if need_print:
        print (
        "Correct bets for home={} | favorite={} | dog={}".format(percent_on_home, percent_on_favorite, percent_on_dog))
        # print ("Net investment:", net_investment)
        print ("ROI for home={} | favorite={} | dog={}".format(roi_on_home, roi_on_favorite, roi_on_dog))
        print ("PRE for home={} | favorite={} | dog={}:".format(pre_on_home, pre_on_favorite, pre_on_dog))
    return pre_on_home, pre_on_favorite, pre_on_dog


def generate_stats_over_all_seasons(match_info, strategy):
    profitablity = {}
    stats_on_home = {}
    stats_on_favorite = {}
    stats_on_dog = {}

    for league in match_info['LEAGUE'].unique():
        for season in match_info['SEASON'].unique():
            #         print('Processing -> {} - {}'.format(league, season))
            match_df = match_info[
                (match_info['LEAGUE'] == league) &
                (match_info['SEASON'] == season)
                ]
            match_df = apply_finance_strategy(match_df, strategy=strategy)
            on_home, on_favorite, on_dog = get_stats_from_matches(match_df)
            profitablity.setdefault(league, {}).setdefault(
                season,
                {
                    'BET_ON_HOME': on_home,
                    'BET_ON_FAVORITE': on_favorite,
                    'BET_ON_DOG': on_dog
                }
            )
            stats_on_home.setdefault(league, {}).setdefault(season, on_home)
            stats_on_favorite.setdefault(league, {}).setdefault(season, on_favorite)
            stats_on_dog.setdefault(league, {}).setdefault(season, on_dog)

    return profitablity, stats_on_home, stats_on_favorite, stats_on_dog


def get_standings_table(matches, n_teams=15):
    home_standings = matches[['HOME_TEAM', 'RESULT']].groupby(by=['HOME_TEAM', 'RESULT']).size().unstack(fill_value=0)
    home_standings.loc[:, 'HOME_SCORE'] = home_standings[['H', 'D', 'A']].apply(lambda x: x[0] * 3 + x[1],
                                                                                axis='columns')
    home_standings.loc[:, 'HOME_MAX_SCORE'] = home_standings[['H', 'D', 'A']].apply(
        lambda x: x[0] * 3 + x[1] * 3 + x[2] * 3, axis='columns')
    away_standings = matches[['AWAY_TEAM', 'RESULT']].groupby(by=['AWAY_TEAM', 'RESULT']).size().unstack(fill_value=0)
    away_standings.loc[:, 'AWAY_SCORE'] = away_standings[['H', 'D', 'A']].apply(lambda x: x[2] * 3 + x[1],
                                                                                axis='columns')
    away_standings.loc[:, 'AWAY_MAX_SCORE'] = away_standings[['H', 'D', 'A']].apply(
        lambda x: x[0] * 3 + x[1] * 3 + x[2] * 3, axis='columns')

    standings = pd.concat([home_standings, away_standings], axis='columns')
    standings.loc[:, 'SCORE'] = standings['HOME_SCORE'] + standings['AWAY_SCORE']
    standings.loc[:, 'MAX_SCORE'] = standings['HOME_MAX_SCORE'] + standings['AWAY_MAX_SCORE']
    standings.loc[:, 'WIN_RATE'] = standings['SCORE'] / standings['MAX_SCORE']
    standings = standings.sort_values(by='WIN_RATE', ascending=False).head(n_teams)

    return standings



#!/usr/bin/env python
import os

from setuptools import setup, find_packages


install_requires = [
    'setuptools',
    'numpy',
    'pandas',
    'requests',
    'openpyxl',
    'seaborn'
]

dependency_links = [
    
]

setup(name='betalytics',
      version=0.3,
      description='DataScience Framework For Betting Analytics',
      author='',
      author_email='',
      url='https://github.com/sashml/betalytics',
      packages=find_packages(),
      install_requires=install_requires,
      dependency_links=dependency_links,
      package_data={'': ['*.yaml']},
      entry_points={
          'console_scripts':
              [
              ],
      },
      )



BOOKIE = {
    'BET365': {
        'B365H': 'HOME_ODDS',
        'B365D': 'DRAW_ODDS',
        'B365A': 'AWAY_ODDS'
    },
    'PS': {
        'PSH': 'HOME_ODDS',
        'PSD': 'DRAW_ODDS',
        'PSA': 'AWAY_ODDS'
    },
    'WH': {
        'WHH': 'HOME_ODDS',
        'WHD': 'DRAW_ODDS',
        'WHA': 'AWAY_ODDS'
    },
    'BWIN': {
        'BWH': 'HOME_ODDS',
        'BWD': 'DRAW_ODDS',
        'BWA': 'AWAY_ODDS'
    },
    'LB': {
        'LBH': 'HOME_ODDS',
        'LBD': 'DRAW_ODDS',
        'LBA': 'AWAY_ODDS'
    }
}


MATCH_INFO = [
    'DATE', 'SEASON', 'COUNTRY', 'LEAGUE', 'DIV',
    'HOMETEAM', 'AWAYTEAM', 'FTAG', 'FTHG', 'FTR'
]